/*
 *  MicroEmulator
 *  Copyright (C) 2003 mpowerplayer inc.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.barteo.cldc.https;

import javax.net.ssl.HttpsURLConnection;
import java.security.cert.X509Certificate;
import javax.microedition.io.HttpConnection;
import javax.microedition.io.HttpsConnection;
import javax.microedition.io.SecurityInfo;
import javax.microedition.pki.Certificate;

public class Connection 
    extends com.barteo.cldc.http.Connection 
    implements HttpsConnection
{  
    public SecurityInfo getSecurityInfo()
    {
        return new SecurityInfoImpl();
    }
    
    private class SecurityInfoImpl implements SecurityInfo
    {
        public String getProtocolName()
        {
            return "SSL"; //FIXME: no clear way to obtain this?
        }
        
        public String getProtocolVersion()
        {
            return "3.0"; // required version for "SSL" per spec
        }
        
        public String getCipherSuite()
        {
            return ((HttpsURLConnection)cn).getCipherSuite();
        }
        
        public Certificate getServerCertificate()
        {
            try
            {
                java.security.cert.Certificate[] certs =
                    ((HttpsURLConnection)cn).getServerCertificates();
                return new CertificateImpl( (X509Certificate) certs[0] );
            }
            catch ( Throwable t )
            {   
                t.printStackTrace();
            }
            return null;
        }
    }
    
    private class CertificateImpl implements Certificate
    {
        X509Certificate cert;
        
        public CertificateImpl( X509Certificate inCert )
        {
            cert = inCert;
        }
        
        public String getSubject()
        {
            return cert.getSubjectDN().getName();
        }
        
        public String getIssuer()
        {
            return cert.getIssuerDN().getName();
        }
        
        public String getType()
        {
            return cert.getType();
        }
        
        public String getVersion()
        {
            return Integer.toString( cert.getVersion() );
        }
        
        public String getSigAlgName()
        {
            return cert.getSigAlgName();
        }
        
        public long getNotBefore()
        {
            return cert.getNotBefore().getTime();
        }
        
        public long getNotAfter()
        {
            return cert.getNotAfter().getTime();
        }
        
        public String getSerialNumber()
        {
            return cert.getSerialNumber().toString();
        }
    }
}
  
