/*
 *  WMA Support for MicroEmulator
 *  Copyright (C) 2005 John Montgomery <john@sensatus.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.barteo.cldc.sms;

import javax.wireless.messaging.*;
//import javax.microedition.io.Connection;

import java.net.*;

import java.io.ByteArrayOutputStream;
import java.io.InterruptedIOException;
import java.io.IOException;

import java.util.Vector;
import java.util.regex.*;

public class Connection implements MessageConnection, Runnable {
	private String localNumber = null;
	private String number = null;
	private String port   = null;
	private InetAddress hostAddress = null;
	private DatagramSocket socket = null;
	private int receivePort = -1;
	private int sendPort    = -1;
	
	private MessageListener messageListener = null;
	
	private Object receiveLock = new Object();
	private Vector receivedMessages = new Vector();
	
	private boolean listening = false;
	
	public javax.microedition.io.Connection open( String name ) throws IOException {
		hostAddress = InetAddress.getByName( System.getProperty( "com.barteo.cldc.sms.DatagramHost", "localhost" ) );
		receivePort = Integer.parseInt( System.getProperty( "com.barteo.cldc.sms.DatagramPortIn", "54321" ) );
		sendPort    = Integer.parseInt( System.getProperty( "com.barteo.cldc.sms.DatagramPortOut", "12345" ) );
		localNumber = System.getProperty( "com.barteio.cldc.sms.SMSNumber", "+555555" );
		
		String[] parts = name.split(":");
		if ( parts.length != 3 )
			throw new RuntimeException( "Expected number and port in: " + name );
		number = parts[1];
		if ( number.startsWith("//") )
			number = number.substring( 2 );
		port   = parts[2];
		
		
		
		// we have created a server MessageConnection
		if ( number.equals("") ) {
			listening = true;
			socket = new DatagramSocket( receivePort, hostAddress );
			System.out.println( "Listening" );
			Thread receiveDaemon = new Thread( this );
			receiveDaemon.setDaemon( true );
			receiveDaemon.start();
		}
		else {
			listening = false;
			socket = new DatagramSocket();
		}
		if ( number.equals("") )
			number = localNumber;
		return this;
	}

	private String address() {
		return "sms://" + number + ":" + port;
	}
	
	private String localAddress() {
		return "sms://" + localNumber + ":" + port;
	}

	public void run() {
		while( socket != null ) {
			DatagramPacket packet = new DatagramPacket(new byte[1024],1024);
			try {
				socket.receive( packet );
				packetReceived( packet );
			}
			catch( IOException ioe ) {
				ioe.printStackTrace();
			}
		}
	}
	
	private void packetReceived( DatagramPacket packet ) {
		//System.out.println("Packet Received");
		synchronized( receiveLock ) {
			String contents = new String( packet.getData(), packet.getOffset(), packet.getLength() );
			Message msg = parseMessage(contents);
			if ( msg != null ) { //might not be a message for us
				System.out.println( "received message" );
				receivedMessages.addElement( msg );
				if ( messageListener != null )
					messageListener.notifyIncomingMessage( this );
			}
		}
	}
	
	private String retrieveValue( String contents, String key ) {
		//System.out.println(key);
		Pattern p = Pattern.compile( "^"+key+":(.*?)$", Pattern.DOTALL | Pattern.MULTILINE );
		//System.out.println(p.pattern());
		Matcher m = p.matcher(contents);
		if ( m.find() ) {
			//System.out.println( key + ":" + m.group(1) );
			return m.group( 1 ).trim();
		}
		//System.out.println( "didn't match" );
		return null;
	}
	
	private Message parseMessage( String content ) {
		System.out.println( content );
		String address = retrieveValue( content, "Address" );
		System.out.println( address + "==" + localAddress() );
		if ( !localAddress().equals( address ) ) {
			return null; // not meant for us
		}
		String type   = retrieveValue( content, "Content-Type" );
		//System.out.println( type );
		String length = retrieveValue( content, "Content-Length" );
		//System.out.println( length );
		
		if ( type == null || length == null )
			return null;
		
		int len = Integer.parseInt( length );
		
		int index = content.indexOf("\nBuffer:\n") + "\nBuffer:\n".length();
		String data = content.substring(index,index+len );
		
		String serverAddress = retrieveValue( content, "ServerAddress" );
		Message msg = newMessage(type,serverAddress);
		
		if ( type.equals( BINARY_MESSAGE ) ) {
			BinaryMessage binMsg = (BinaryMessage)msg;
			binMsg.setPayloadData( data.getBytes() );
		}
		else if ( type.equals( TEXT_MESSAGE ) ) {
			TextMessage txtMsg = (TextMessage)msg;
			txtMsg.setPayloadText(data);
		}
		
		return msg;
	}

	public Message newMessage(String type) {
		String address = null;
		if ( !listening ) // not listening, so must be client socket
			address = address();
		return newMessage(type,address);
	}
	
	public Message newMessage(String type,String address) {
		Message msg = null;
		if ( TEXT_MESSAGE.equals( type ) ) {
			msg = new TextMessageImpl();
		}
		else if ( BINARY_MESSAGE.equals( type ) ) {
			msg = new BinaryMessageImpl();
		}
		msg.setAddress(address);
		return msg;
	}

	public int numberOfSegments(Message msg) {
		// TODO limit max size of messages
		return 1;
	}

	public Message receive() throws IOException, InterruptedIOException {
		synchronized( receiveLock ) {
			while( receivedMessages.size() == 0 ) {
				try {
					receiveLock.wait( 500 );
				}
				catch( InterruptedException ie ) {
					ie.printStackTrace();
				}	
			}
			Message msg = (Message)receivedMessages.elementAt( 0 );
			receivedMessages.removeElementAt( 0 );
			return msg;
		}
	}

	public void send(Message msg) throws IOException, InterruptedIOException {
		String address = msg.getAddress();
		if ( address == null )
			throw new IllegalArgumentException("No target address specified for message" );
		byte[] contents = prepMessage( msg );
		DatagramPacket packet = new DatagramPacket( contents, contents.length, hostAddress, sendPort );
		socket.send( packet );
	}
	
	private byte[] prepMessage(Message msg) throws IOException {
		String type = "";
		byte[] data = null;
		if ( msg instanceof TextMessage ) {
			type = TEXT_MESSAGE;
			data = ((TextMessage)msg).getPayloadText().getBytes();
		}
		else {
			type = BINARY_MESSAGE;
			data = ((BinaryMessage)msg).getPayloadData();
		}
		StringBuffer buffer = new StringBuffer();
		buffer.append( "Text-Encoding: gsm7bit\n" );
		buffer.append( "Content-Type: " + type + "\n" );
		buffer.append( "Content-Length: " + data.length + "\n" );
		buffer.append( "Segments: 1\n" );
		buffer.append( "SenderAddress: " + localAddress() + "\n" );
		buffer.append( "Address: " + msg.getAddress() + "\n" );
		buffer.append( "Date: " + msg.getTimeStamp().getTime() + "\n" );
		buffer.append( "Buffer:\n" );
		buffer.append( new String(data) );
		
		return buffer.toString().getBytes();
	}

	public void setMessageListener(MessageListener l) {
		this.messageListener = l;
	}
	
	public void close() throws IOException {
		if ( socket != null )
			socket.close();
		socket = null;
	}
	
}