/*
 *  MicroEmulator
 *  Copyright (C) 2001-2003 Bartek Teodorczyk <barteo@it.pl>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.barteo.cldc.socket;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;

import javax.microedition.io.SocketConnection;

import com.barteo.cldc.ClosedConnection;


public class Connection implements SocketConnection, ClosedConnection 
{
	private Socket socket;


	public javax.microedition.io.Connection open(String name)
		throws IOException
	{
		int portSepIndex = name.lastIndexOf(':');
		int port = Integer.parseInt(name.substring(portSepIndex + 1));		
		String host = name.substring("socket://".length(), portSepIndex);

		socket = new Socket(host, port);
    
	  return this;
	}


	public void close() 
			throws IOException 
	{
		if (socket == null) {
			return;
		}
		
		socket.close();
	}


	public InputStream openInputStream() 
			throws IOException 
	{
		if (socket == null) {
		  throw new IOException();
		}
		
		return socket.getInputStream();
	}


	public DataInputStream openDataInputStream() 
			throws IOException 
	{
		return new DataInputStream(openInputStream());
	}


	public OutputStream openOutputStream() 
			throws IOException 
	{
		if (socket == null) {
		  throw new IOException();
		}

		return socket.getOutputStream();
	}


	public DataOutputStream openDataOutputStream() 
			throws IOException 
	{
		return new DataOutputStream(openOutputStream());
	}
    
    public String getLocalAddress() throws IOException
    {
        return socket.getLocalAddress().getHostAddress();
    }

    public int getLocalPort() throws IOException
    {
        return socket.getLocalPort();
    }

    public String getAddress() throws IOException
    {
        return socket.getInetAddress().getHostAddress();
    }

    public int getPort() throws IOException
    {
        return socket.getPort();
    }

    public void setSocketOption(byte option, int value) 
        throws IllegalArgumentException, IOException
    {
        if ( value < 0 ) 
        {
            throw new IllegalArgumentException(
                "Option value cannot be less than zero: " + value );
        }
            
        switch ( option )
        {
            case DELAY: // double-negative confusing 
                socket.setTcpNoDelay( value==0 ); 
                break;
            case LINGER:
                socket.setSoLinger( value!=0, value );
                break;
            case KEEPALIVE:
                socket.setKeepAlive( value!=0 ); 
                break;
            case RCVBUF:
                socket.setReceiveBufferSize( value );
                break;
            case SNDBUF:
                socket.setSendBufferSize( value );
                break;
            default:
                throw new IllegalArgumentException(
                    "Invalid option identifier: " + option );
        }
    }

    public int getSocketOption(byte option) 
        throws IllegalArgumentException, IOException
    {
        switch ( option )
        {
            case DELAY:
                return socket.getTcpNoDelay() ? 0 : 1;
            case LINGER:
                int linger = socket.getSoLinger();
                if ( linger == -1 ) linger = 0;
                return linger;
            case KEEPALIVE:
                return socket.getKeepAlive() ? 1 : 0;
            case RCVBUF:
                return socket.getReceiveBufferSize();
            case SNDBUF:
                return socket.getSendBufferSize();
            default:
                throw new IllegalArgumentException(
                    "Invalid option identifier: " + option );
        }
    }

}
