/*
 *  MicroEmulator
 *  Copyright (C) 2001 Bartek Teodorczyk <barteo@it.pl>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
package javax.microedition.io;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

public class Connector
{

	public static final int READ = 1;
	public static final int WRITE = 2;
	public static final int READ_WRITE = 3;
	
	
	public static Connection open(String name)
      throws IOException
  {
    return open(name, READ_WRITE, false);
  }
  
  
  public static Connection open(String name, int mode)
      throws IOException
  {
    return open(name, mode, false);
  }
  
    // this method Copyright (C) 2005 Just van den Broecke <just AT justobjects.nl>
    public static Connection open(String name, int mode, boolean timeouts) throws IOException {

        String url = name.trim().toLowerCase();
        //System.err.println("connbridge: opening " + name);

        // use reflection as not  to be directly dependent

        try {
            // Divert Bluetooth URL to Avetana Bluetooth impl.
            if (url.startsWith("bt") || url.startsWith("localhost")) {
                Class cl = Class.forName("de.avetana.bluetooth.connection.Connector");
                Class[] parms = {String.class, int.class, boolean.class};
                Method openMethod = cl.getDeclaredMethod("open", parms);
                Object[] args = {name, new Integer(mode), new Boolean(timeouts)};
                return (Connection) openMethod.invoke(null, args);

            } else {
                // Standard URL like socket:// or http://
                // Use MPP-related Connector implementation
                Class cl = Class.forName(
                        "com.barteo.cldc." + name.substring(0, name.indexOf(':')) + ".Connection");
                Object conn = cl.newInstance();
                Class[] parms = {String.class};
                Method openMethod = cl.getDeclaredMethod("open", parms);
                Object[] args = {name};
                return (Connection) openMethod.invoke(conn, args);
            }
        } catch (ClassNotFoundException ex) {
            System.err.println(ex);
            throw new ConnectionNotFoundException();
        } catch (InstantiationException ex) {
            System.err.println(ex);
            throw new ConnectionNotFoundException();
        } catch (NoSuchMethodException ex) {
            System.err.println(ex);
            throw new ConnectionNotFoundException();
        } catch (IllegalAccessException ex) {
            System.err.println(ex);
            throw new ConnectionNotFoundException();
        } catch (InvocationTargetException ex) {
            System.err.println(ex.getTargetException());
            throw new ConnectionNotFoundException();
        }

    }
  
  public static DataInputStream openDataInputStream(String name)
      throws IOException
  {
    InputConnection cn = (InputConnection) open(name);
    
    return cn.openDataInputStream();
  }
  
  
  public static DataOutputStream openDataOutputStream(String name)
      throws IOException
  {
    OutputConnection cn = (OutputConnection) open(name);
    
    return cn.openDataOutputStream();
  }
  
  
  public static InputStream openInputStream(String name)
      throws IOException
  {
    InputConnection cn = (InputConnection) open(name);
    
    return cn.openInputStream();
  }
  
    
	public static OutputStream openOutputStream(String name)
      throws IOException
  {
    OutputConnection cn = (OutputConnection) open(name);
    
    return cn.openOutputStream();
  }
  
}
